<?php
/**
 * @package     JLex Comment
 * @subpackage  Search plugin 
 * @version     1.0.0
 * @copyright   Copyright (c) 2013-2018 JLexArt. All rights reserved.
 * @license     GNU/GPL or later
 * @author      JLexArt (https://www.jlexart.com/)
 */

defined('_JEXEC') or die;

class plgSearchJLexComment extends JPlugin
{

    protected $autoloadLanguage = true;

    public function onContentSearchAreas()
    {
        static $areas = array(
            'cm' => 'PLG_SEARCH_COMMENTS'
        );
        
        return $areas;
    }

    public function onContentSearch($text, $phrase='', $ordering='', $areas=null)
    {
        $app    = JFactory::getApplication();
        $db     = JFactory::getDbo();
        $user   = JFactory::getUser();
        $now    = JFactory::getDate()->toSql();
        $loader = JPATH_ROOT . '/components/com_jlexcomment/bootstrap.php';

        if(!is_file($loader))
        {
            return array();
        }
        
        if (is_array($areas))
        {
            if (! array_intersect($areas, array_keys($this->onContentSearchAreas())))
            {
                return array();
            }
        }
        
        $limit = $this->params->def('limit', 20);
        $limit = $limit>1 ? $limit*1 : 20;
        $query = $db->getQuery(true);

        if(preg_match('/^\s*$/', $text)) return array();

        $query->clear()
              ->select('cm.*, e.title, e.com_name, e.com_key, e.url')
              ->from('#__jlexcomment cm')
              ->leftJoin('#__jlexcomment_obj e ON cm.obj_id=e.id')
              ->where(array(
                    'e.published=1',
                    'cm.published=1',
                    'e.id IS NOT NULL'
                ))
              ->group('cm.id');


        $whereClauses = array();
        
        switch ($phrase)
        {
            case 'exact':
                $safeStr    = $db->quote('%' . $db->escape($text, true) . '%', false);
                $orClauses  = array(
                        'cm.comment LIKE ' . $safeStr
                    );

                $whereClauses[] = '(' . implode(') OR (', $orClauses) . ')';
                break;
            
            case 'all':
            case 'any':
            default:
                $words = explode(' ', $text);
                $whereChild = array();
                
                foreach ($words as $word) {
                    $word = $db->quote('%' . $db->escape($word, true) . '%', false);
                    
                    $orClauses = array(
                            'cm.comment LIKE ' . $word,
                        );

                    $whereChild[] = implode(' OR ', $orClauses);
                }
                
                $whereClauses[] = '(' . implode(($phrase == 'all' ? ') AND (' : ') OR ('), $whereChild) . ')';
                break;
        }
        
        switch ($ordering) {
            case 'oldest':
                $order = 'cm.created_time ASC';
                break;
            
            case 'popular':
                $order = 'cm.child_count_active DESC';
                break;
            
            case 'alpha':
                $order = 'cm.comment ASC';
                break;
            
            case 'category':
            case 'newest':
            default:
                $order = 'cm.created_time DESC';
                break;
        }

        $query->where($whereClauses)
              ->order($order);
        
        $rows       = null;
        $results    = array();

        $db->setQuery($query, 0, $limit);
        try {
            $rows = $db->loadObjectList();
        } catch (RuntimeException $e) {
            JFactory::getApplication()->enqueueMessage(JText::_('JERROR_AN_ERROR_HAS_OCCURRED'), 'error');
        }

        if(!$rows) return $results;

        // load config
        include_once $loader;
        $config  = JLexCommentHelper::getConfig();  

        foreach ($rows as $row)
        {
            // find object
            if ($config->get('cm_link',0)=='1')
            {
                $up = CommentHelperAdmin::getObjDetail($row->com_name, $row->com_key, $row->title, $row->url); 
                
                $row->url     = $up->url;
                $row->title   = $up->title;
            } else {
                $row->url  = JUri::base(true) . '/' . trim(urldecode($row->url),'/');
            }


            $item = new stdClass();
            $item->text     = strip_tags($row->comment);
            $item->title    = $row->title;
            $item->href     = $row->url . (preg_match('/\?/', $row->url)?'&':'?') . 'comment_id=' . $row->id;
            $item->created  = $row->created_time;

            // sub variable
            $item->section  = null;
            $item->browsernav = null;

            $results[] = $item;
        }

        return $results;
    }
}
